"""Hotfix Image.tag and CMFImage.tag to use PIL to size images (useful for thumbnails).  Also fixes so-called intelligent scaling"""


import string
import PIL.Image
from cStringIO import StringIO
import OFS.Image
from webdav.common import rfc1123_date
from DateTime import DateTime

__ac_permissions__=(
    ('View management screens',
     ('manage', 'manage_main',)),
    ('Change Images and Files',
     ('manage_edit','manage_upload','PUT')),
    ('View',
     ('index_html', 'tag', 'view_image_or_file', 'get_size',
      'getContentType', '', 'thumb')),
    ('FTP access',
     ('manage_FTPstat','manage_FTPget','manage_FTPlist')),
    ('Delete objects',
     ('DELETE',)),
    )



def thumb(self, REQUEST, RESPONSE, height=None, width=None):
    """
    Makes and returns a thumbnail image with PIL
    """
    
    if not(width or height):
        width = 80
        height = 80
    keys = {'height':int(width or height), 'width':int(height or width)} #make sure we have valid int's
    
    thumbnail_file = self.ZCacheable_get(view_name='thumb', keywords=keys)   
    if (not thumbnail_file) or (not self.ZCacheable_enabled()):
        #create a thumbnail image file
        original_file=StringIO(str(self.data)) #use str() in case data is Pdata
        image=PIL.Image.open(original_file)
        image=image.convert('RGB')
        image.thumbnail((keys['width'],keys['height'])) 
        thumbnail_file=StringIO()
        image.save(thumbnail_file, "JPEG")
    thumbnail_file.seek(0,2)
    thumbnail_size = thumbnail_file.tell()
    thumbnail_file.seek(0)
            
    # HTTP If-Modified-Since header handling.
    header=REQUEST.get_header('If-Modified-Since', None)
    if header is not None:
        header=string.split(header, ';')[0]
        # Some proxies seem to send invalid date strings for this
        # header. If the date string is not valid, we ignore it
        # rather than raise an error to be generally consistent
        # with common servers such as Apache (which can usually
        # understand the screwy date string as a lucky side effect
        # of the way they parse it).
        try:    mod_since=long(DateTime(header).timeTime())
        except: mod_since=None
        if mod_since is not None:
            if self._p_mtime:
                last_mod = long(self._p_mtime)
            else:
                last_mod = long(0)
            if last_mod > 0 and last_mod <= mod_since:
                # Set header values since apache caching will return Content-Length
                # of 0 in response if size is not set here
                RESPONSE.setHeader('Last-Modified', rfc1123_date(self._p_mtime))
                RESPONSE.setHeader('Content-Type', 'image/jpeg')
                RESPONSE.setHeader('Content-Length', thumbnail_size)
                RESPONSE.setStatus(304)
                return ''

    RESPONSE.setHeader('Last-Modified', rfc1123_date(self._p_mtime))
    RESPONSE.setHeader('Content-Type', 'image/jpeg')
    RESPONSE.setHeader('Content-Length', thumbnail_size)

    # Don't cache the data itself, but provide an opportunity
    # for a cache manager to set response headers.
    #WE WANT TO CHANGE THIS TO CACHE THE THUMBS
    if thumbnail_file:
        self.ZCacheable_set(None)
    else:
        self.ZCacheable_set(thumbnail_file, view_name='thumb', keywords=keys, mtime_func=None)   

    return thumbnail_file.read()



def tag(self, height=None, width=None, alt=None,
        scale=0, xscale=0, yscale=0, css_class=None, **args):
    """
    Generate an HTML IMG tag for this image, with customization.
    Arguments to self.tag() can be any valid attributes of an IMG tag.
    'src' will always be an absolute pathname, to prevent redundant
    downloading of images. Defaults are applied intelligently for
    'height', 'width', and 'alt'. If specified, the 'scale', 'xscale',
    and 'yscale' keyword arguments will be used to automatically adjust
    the output height and width values of the image tag.

    Since 'class' is a Python reserved word, it cannot be passed in
    directly in keyword arguments which is a problem if you are
    trying to use 'tag()' to include a CSS class. The tag() method
    will accept a 'css_class' argument that will be converted to
    'class' in the output tag to work around this.
    """
    #don't do this -- we want to know later if the user spec'd these or not
    #if height is None: height=self.height
    #if width is None:  width=self.width

    # Auto-scaling support
    xdelta = xscale or scale
    ydelta = yscale or scale

    if xdelta:
        width =  str(int(round(int(width or self.width) * xdelta)))
    if ydelta:
        height = str(int(round(int(height or self.height) * ydelta)))
    
    #see if params are really int's quoted as strings, or something else like %'s
    if height:
	try:
	    height = int(height)
	except:
	    pass    
    if width:
	try:
	    width = int(width)
	except:
	    pass    

    #do PIL thumb generation only if heigth or width spec are int's
    if ((height and (type(height) is type(0))) or
        (width and (type(width) is type(0)))):
                
        result='<img src="%s/thumb?' % (self.absolute_url())

        if height:
            result = '%sheight=%s' % (result, height)
            
        if height and width:
            result = '%s&' % result

        if width:
            result = '%swidth=%s' % (result, width)
            
        result='%s"' % result
            
            
    else:    
        result='<img src="%s"' % (self.absolute_url())

        if height:
            result = '%s height="%s"' % (result, height)

        if width:
            result = '%s width="%s"' % (result, width)

    if alt is None:
        alt=getattr(self, 'title', '')
    if alt == '':
	alt=str(self.getId())
    result = '%s alt="%s"' % (result, alt)

    if not 'border' in map(string.lower, args.keys()):
        result = '%s border="0"' % result

    if css_class is not None:
        result = '%s class="%s"' % (result, css_class)

    for key in args.keys():
        value = args.get(key)
        result = '%s %s="%s"' % (result, key, value)

    return '%s />' % result

#now dynamically override/add the methods
OFS.Image.Image.tag = tag
OFS.Image.Image.thumb = thumb
OFS.Image.Image.__ac_permissions__ = __ac_permissions__
